@@ -1,7 +1,7 @@
 import lief
 import sys
 
-pe = lief.parse(sys.argv[1])
+pe = lief.PE.parse(sys.argv[1])
 exports = pe.get_export()
 
 for e in filter(lambda e: e.is_forwarded, exports.entries):
