@@ -1,5 +1,5 @@
-/* Copyright 2017 - 2023 R. Thomas
- * Copyright 2017 - 2023 Quarkslab
+/* Copyright 2017 - 2024 R. Thomas
+ * Copyright 2017 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,23 +13,27 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#include "pyPlatform.hpp"
+#include "platforms/pyPlatform.hpp"
+#include "LIEF/platforms.hpp"
 #include "enums_wrapper.hpp"
+#include "pyLIEF.hpp"
+#include "platforms/android/pyAndroid.hpp"
 
-namespace LIEF {
+namespace LIEF::py {
 
-void init_python_platforms(py::module& m) {
+void init_platforms(nb::module_& m) {
   LIEF::enum_<PLATFORMS>(m, "PLATFORMS")
-    .value("UNKNOWN", PLATFORMS::UNKNOWN)
-    .value("LINUX",   PLATFORMS::LINUX)
-    .value("ANDROID", PLATFORMS::ANDROID_PLAT)
-    .value("WINDOWS", PLATFORMS::WINDOWS)
-    .value("IOS",     PLATFORMS::IOS)
-    .value("OSX",     PLATFORMS::OSX);
+    .value("UNKNOWN", PLATFORMS::PLAT_WINDOWS)
+    .value("LINUX",   PLATFORMS::PLAT_LINUX)
+    .value("ANDROID", PLATFORMS::PLAT_ANDROID)
+    .value("WINDOWS", PLATFORMS::PLAT_WINDOWS)
+    .value("IOS",     PLATFORMS::PLAT_IOS)
+    .value("OSX",     PLATFORMS::PLAT_OSX);
 
   m.def("current_platform", &current_platform,
-      "Return the current plaform (Linux, Windows, ...) as a :attr:`lief.PLATFORMS` enum");
+      "Return the current plaform (Linux, Windows, ...) as a :attr:`lief.PLATFORMS` enum"_doc);
 
+  LIEF::Android::py::init_module(m);
 }
 
 }
