@@ -1,5 +1,5 @@
-/* Copyright 2017 - 2023 R. Thomas
- * Copyright 2017 - 2023 Quarkslab
+/* Copyright 2017 - 2024 R. Thomas
+ * Copyright 2017 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,10 +14,20 @@
  * limitations under the License.
  */
 #include "LIEF/Object.hpp"
+#include "LIEF/hash.hpp"
 
 namespace LIEF {
 Object::Object() = default;
 Object::Object(const Object&) = default;
 Object& Object::operator=(const Object&) = default;
 Object::~Object() = default;
+
+bool Object::operator==(const Object& other) const {
+  if (this == &other) {
+    return true;
+  }
+
+  return hash(*this) == hash(other);
+}
+
 }
