@@ -1,5 +1,5 @@
-/* Copyright 2017 - 2023 R. Thomas
- * Copyright 2017 - 2023 Quarkslab
+/* Copyright 2017 - 2024 R. Thomas
+ * Copyright 2017 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,23 +24,26 @@
 #include "LIEF/BinaryStream/VectorStream.hpp"
 #include "LIEF/PE/signature/Signature.hpp"
 #include "LIEF/PE/signature/SignatureParser.hpp"
-#include "LIEF/PE/CodeViewPDB.hpp"
-#include "LIEF/PE/Parser.hpp"
-#include "LIEF/PE/utils.hpp"
-#include "LIEF/PE/Section.hpp"
 #include "LIEF/PE/Binary.hpp"
 #include "LIEF/PE/DataDirectory.hpp"
+#include "LIEF/PE/EnumToString.hpp"
+#include "LIEF/PE/Export.hpp"
+#include "LIEF/PE/ExportEntry.hpp"
+#include "LIEF/PE/Parser.hpp"
+#include "LIEF/PE/debug/CodeViewPDB.hpp"
+#include "LIEF/PE/debug/Pogo.hpp"
+#include "LIEF/PE/debug/Repro.hpp"
+#include "LIEF/PE/debug/PogoEntry.hpp"
+#include "LIEF/PE/Relocation.hpp"
+#include "LIEF/PE/RelocationEntry.hpp"
 #include "LIEF/PE/ResourceData.hpp"
 #include "LIEF/PE/ResourceDirectory.hpp"
 #include "LIEF/PE/ResourceNode.hpp"
-#include "LIEF/PE/Export.hpp"
-#include "LIEF/PE/ExportEntry.hpp"
-#include "LIEF/PE/Pogo.hpp"
-#include "LIEF/PE/PogoEntry.hpp"
-#include "LIEF/PE/Relocation.hpp"
-#include "LIEF/PE/RelocationEntry.hpp"
+#include "LIEF/PE/RichHeader.hpp"
+#include "LIEF/PE/Section.hpp"
 #include "LIEF/PE/Symbol.hpp"
-#include "LIEF/PE/EnumToString.hpp"
+#include "LIEF/PE/TLS.hpp"
+#include "LIEF/PE/utils.hpp"
 
 #include "internal_utils.hpp"
 #include "Parser.tcc"
@@ -54,15 +57,9 @@
 namespace LIEF {
 namespace PE {
 
-constexpr size_t Parser::MAX_PADDING_SIZE;
-constexpr size_t Parser::MAX_TLS_CALLBACKS;
-constexpr size_t Parser::MAX_DLL_NAME_SIZE;
-constexpr size_t Parser::MAX_DATA_SIZE;
-
 Parser::~Parser() = default;
 Parser::Parser() = default;
 
-
 Parser::Parser(const std::string& file) :
   LIEF::Parser{file}
 {
@@ -77,29 +74,29 @@ Parser::Parser(std::vector<uint8_t> data) :
   Parser{std::make_unique<VectorStream>(std::move(data))}
 {}
 
-
-Parser::Parser(std::unique_ptr<BinaryStream> stream) : stream_{std::move(stream)}
+Parser::Parser(std::unique_ptr<BinaryStream> stream) :
+  stream_{std::move(stream)}
 {}
 
-
-void Parser::init(const std::string& name) {
+void Parser::init(const ParserConfig& config) {
   stream_->setpos(0);
   auto type = get_type_from_stream(*stream_);
   if (!type) {
     LIEF_ERR("Can't determine PE type.");
     return;
   }
+
   type_   = type.value();
   binary_ = std::unique_ptr<Binary>(new Binary{});
-  binary_->name(name);
   binary_->type_ = type_;
+  binary_->original_size_ = stream_->size();
+  config_ = config;
 
   if (type_ == PE_TYPE::PE32) {
     parse<details::PE32>();
   } else {
     parse<details::PE64>();
   }
-
 }
 
 ok_error_t Parser::parse_dos_stub() {
@@ -126,26 +123,27 @@ ok_error_t Parser::parse_rich_header() {
   LIEF_DEBUG("Parsing rich header");
   span<const uint8_t> dos_stub = binary_->dos_stub();
 
-  SpanStream stream(dos_stub);
+  const SpanStream stream(dos_stub);
 
-  const auto it_rich = std::search(std::begin(dos_stub), std::end(dos_stub),
-                                   std::begin(details::Rich_Magic), std::end(details::Rich_Magic));
+  const auto* it_rich = std::search(std::begin(dos_stub), std::end(dos_stub),
+                                    std::begin(details::Rich_Magic), std::end(details::Rich_Magic));
 
   if (it_rich == std::end(dos_stub)) {
     LIEF_DEBUG("Rich header not found!");
     return ok();
   }
+  auto rich_header = std::make_unique<RichHeader>();
 
   const uint64_t end_offset_rich_header = std::distance(std::begin(dos_stub), it_rich);
   LIEF_DEBUG("Offset to rich header: 0x{:x}", end_offset_rich_header);
 
   if (auto res_xor_key = stream.peek<uint32_t>(end_offset_rich_header + sizeof(details::Rich_Magic))) {
-    binary_->rich_header().key(*res_xor_key);
+    rich_header->key(*res_xor_key);
   } else {
     return make_error_code(lief_errors::read_error);
   }
 
-  const uint32_t xor_key = binary_->rich_header().key();
+  const uint32_t xor_key = rich_header->key();
   LIEF_DEBUG("XOR key: 0x{:x}", xor_key);
 
   int64_t curent_offset = end_offset_rich_header - sizeof(details::Rich_Magic);
@@ -153,8 +151,6 @@ ok_error_t Parser::parse_rich_header() {
   std::vector<uint32_t> values;
   values.reserve(dos_stub.size() / sizeof(uint32_t));
 
-  result<uint32_t> res_count = 0;
-  result<uint32_t> res_value = 0;
   uint32_t count = 0;
   uint32_t value;
 
@@ -185,17 +181,17 @@ ok_error_t Parser::parse_rich_header() {
       break;
     }
 
-    uint16_t build_number = value & 0xFFFF;
-    uint16_t id           = (value >> 16) & 0xFFFF;
+    const uint16_t build_number = value & 0xFFFF;
+    const uint16_t id           = (value >> 16) & 0xFFFF;
 
     LIEF_DEBUG("ID:           0x{:04x}", id);
     LIEF_DEBUG("Build Number: 0x{:04x}", build_number);
     LIEF_DEBUG("Count:        0x{:d}", count);
 
-    binary_->rich_header().add_entry(id, build_number, count);
+    rich_header->add_entry(id, build_number, count);
   }
 
-  binary_->has_rich_header_ = true;
+  binary_->rich_header_ = std::move(rich_header);
   return ok();
 }
 
@@ -228,7 +224,7 @@ ok_error_t Parser::parse_sections() {
     }
     auto section = std::make_unique<Section>(raw_sec);
     uint32_t size_to_read = 0;
-    uint32_t offset = raw_sec.PointerToRawData;
+    const uint32_t offset = raw_sec.PointerToRawData;
     if (offset > 0) {
       first_section_offset = std::min(first_section_offset, offset);
     }
@@ -238,7 +234,7 @@ ok_error_t Parser::parse_sections() {
                    raw_sec.SizeOfRawData;
 
     if ((offset + size_to_read) > stream_->size()) {
-      uint32_t delta = (offset + size_to_read) - stream_->size();
+      const uint32_t delta = (offset + size_to_read) - stream_->size();
       size_to_read = size_to_read - delta;
     }
 
@@ -246,7 +242,8 @@ ok_error_t Parser::parse_sections() {
       LIEF_WARN("Data of section section '{}' is too large (0x{:x})", section->name(), size_to_read);
     } else {
 
-      if (!stream_->peek_data(section->content_, offset, size_to_read)) {
+      if (!stream_->peek_data(section->content_, offset, size_to_read,
+                              section->virtual_address())) {
         LIEF_ERR("Section #{:d} ({}) is corrupted", i, section->name());
       }
 
@@ -269,8 +266,9 @@ ok_error_t Parser::parse_sections() {
       }
       uint64_t padding_to_read = padding_size + hole_size;
       if (padding_to_read > Parser::MAX_PADDING_SIZE) {
-        LIEF_WARN("The padding size of section '{}' is huge. Only the first {} bytes will be taken"
-                  " into account", section->name(), Parser::MAX_PADDING_SIZE);
+        LIEF_WARN("The padding size of section '{}' is huge. "
+                  "Only the first {} bytes will be taken "
+                  "into account", section->name(), Parser::MAX_PADDING_SIZE);
         padding_to_read = Parser::MAX_PADDING_SIZE;
       }
 
@@ -292,17 +290,18 @@ ok_error_t Parser::parse_sections() {
 }
 
 
-//
-// parse relocations
-//
 ok_error_t Parser::parse_relocations() {
   static constexpr size_t MAX_RELOCATION_ENTRIES = 100000;
-  LIEF_DEBUG("== Parsing relocations ==");
+  LIEF_DEBUG("Parsing relocations");
 
-  const uint32_t offset = binary_->rva_to_offset(
-      binary_->data_directory(DATA_DIRECTORY::BASE_RELOCATION_TABLE).RVA());
+  const DataDirectory* reloc_dir = binary_->data_directory(DataDirectory::TYPES::BASE_RELOCATION_TABLE);
 
-  const uint32_t max_size = binary_->data_directory(DATA_DIRECTORY::BASE_RELOCATION_TABLE).size();
+  if (reloc_dir == nullptr) {
+    return make_error_code(lief_errors::not_found);
+  }
+
+  const uint32_t offset = binary_->rva_to_offset(reloc_dir->RVA());
+  const uint32_t max_size = reloc_dir->size();
   const uint32_t max_offset = offset + max_size;
 
   auto res_relocation_headers = stream_->peek<details::pe_base_relocation_block>(offset);
@@ -341,7 +340,7 @@ ok_error_t Parser::parse_relocations() {
         LIEF_ERR("Can't parse relocation entry #{}", i);
         break;
       }
-      auto entry = std::make_unique<RelocationEntry>(*res_entry);
+      auto entry = std::make_unique<RelocationEntry>(RelocationEntry::from_raw(this->binary_->header().machine(), *res_entry));
       entry->relocation_ = relocation.get();
       relocation->entries_.push_back(std::move(entry));
     }
@@ -351,18 +350,18 @@ ok_error_t Parser::parse_relocations() {
     res_relocation_headers = stream_->peek<details::pe_base_relocation_block>(current_offset);
   }
 
-  binary_->has_relocations_ = true;
   return ok();
 }
 
-
-//
-// parse ressources
-//
 ok_error_t Parser::parse_resources() {
-  LIEF_DEBUG("== Parsing resources ==");
+  LIEF_DEBUG("Parsing resources");
+  const DataDirectory* res_dir = binary_->data_directory(DataDirectory::TYPES::RESOURCE_TABLE);
 
-  const uint32_t resources_rva = binary_->data_directory(DATA_DIRECTORY::RESOURCE_TABLE).RVA();
+  if (res_dir == nullptr) {
+    return make_error_code(lief_errors::not_found);
+  }
+
+  const uint32_t resources_rva = res_dir->RVA();
   LIEF_DEBUG("Resources RVA: 0x{:04x}", resources_rva);
 
   const uint32_t offset = binary_->rva_to_offset(resources_rva);
@@ -373,18 +372,15 @@ ok_error_t Parser::parse_resources() {
     return make_error_code(lief_errors::read_error);
   }
 
-  binary_->resources_     = parse_resource_node(*res_directory_table, offset, offset);
-  binary_->has_resources_ = binary_->resources_ != nullptr;
+  binary_->resources_ = parse_resource_node(*res_directory_table, offset, offset);
   return ok();
 }
 
-
-//
-// parse the resources tree
-//
-std::unique_ptr<ResourceNode> Parser::parse_resource_node(const details::pe_resource_directory_table& directory_table,
-                                          uint32_t base_offset, uint32_t current_offset, uint32_t depth) {
-
+std::unique_ptr<ResourceNode>
+Parser::parse_resource_node(const details::pe_resource_directory_table& directory_table,
+                            uint32_t base_offset, uint32_t current_offset,
+                            uint32_t depth)
+{
   const uint32_t numberof_ID_entries   = directory_table.NumberOfIDEntries;
   const uint32_t numberof_name_entries = directory_table.NumberOfNameEntries;
 
@@ -434,6 +430,14 @@ std::unique_ptr<ResourceNode> Parser::parse_resource_node(const details::pe_reso
       uint32_t offset = base_offset + data_rva;
       details::pe_resource_data_entry data_entry;
 
+      if (!resource_visited_.insert(offset).second) {
+        if (resource_visited_.size() == 1) {
+          // Only print once
+          LIEF_WARN("Infinite loop detected on resources");
+        }
+        break;
+      }
+
       if (auto res_data_entry = stream_->peek<details::pe_resource_data_entry>(offset)) {
         data_entry = *res_data_entry;
       } else {
@@ -445,7 +449,9 @@ std::unique_ptr<ResourceNode> Parser::parse_resource_node(const details::pe_reso
       uint32_t code_page      = data_entry.Codepage;
 
       std::vector<uint8_t> leaf_data;
-      if (stream_->peek_data(leaf_data, content_offset, content_size)) {
+      if (stream_->peek_data(leaf_data, content_offset, content_size,
+                             data_entry.DataRVA))
+      {
         auto node = std::make_unique<ResourceData>(std::move(leaf_data), code_page);
 
         node->depth_ = depth + 1;
@@ -464,7 +470,10 @@ std::unique_ptr<ResourceNode> Parser::parse_resource_node(const details::pe_reso
       const uint32_t directory_rva = data_rva & (~ 0x80000000);
       const uint32_t offset        = base_offset + directory_rva;
       if (!resource_visited_.insert(offset).second) {
-        LIEF_WARN("Infinite loop detected on resources");
+        if (resource_visited_.size() == 1) {
+          // Only print once
+          LIEF_WARN("Infinite loop detected on resources");
+        }
         break;
       }
 
@@ -488,18 +497,15 @@ std::unique_ptr<ResourceNode> Parser::parse_resource_node(const details::pe_reso
   return directory;
 }
 
-//
-// parse string table
-//
 ok_error_t Parser::parse_string_table() {
-  LIEF_DEBUG("== Parsing string table ==");
+  LIEF_DEBUG("Parsing string table");
   uint32_t string_table_offset =
     binary_->header().pointerto_symbol_table() +
-    binary_->header().numberof_symbols() * details::STRUCT_SIZES::Symbol16Size;
+    binary_->header().numberof_symbols() * details::sizes::SYMBOL_16;
 
   auto res_size = stream_->peek<uint32_t>(string_table_offset);
   if (!res_size) {
-    return res_size.error();
+    return make_error_code(res_size.error());
   }
 
   auto size = *res_size;
@@ -522,11 +528,8 @@ ok_error_t Parser::parse_string_table() {
 }
 
 
-//
-// parse Symbols
-//
 ok_error_t Parser::parse_symbols() {
-  LIEF_DEBUG("== Parsing symbols ==");
+  LIEF_DEBUG("Parsing symbols");
   uint32_t symbol_table_offset = binary_->header().pointerto_symbol_table();
   uint32_t nb_symbols          = binary_->header().numberof_symbols();
   uint32_t current_offset      = symbol_table_offset;
@@ -549,7 +552,7 @@ ok_error_t Parser::parse_symbols() {
     } else {
       uint64_t offset_name =
         binary_->header().pointerto_symbol_table() +
-        binary_->header().numberof_symbols() * details::STRUCT_SIZES::Symbol16Size +
+        binary_->header().numberof_symbols() * details::sizes::SYMBOL_16 +
         raw_symbol.Name.Name.Offset;
       auto res_string = stream_->peek_string_at(offset_name, stream_max_size - offset_name);
       if (res_string) {
@@ -558,8 +561,8 @@ ok_error_t Parser::parse_symbols() {
     }
 
     if (symbol.section_number() > 0 &&
-        static_cast<uint32_t>(symbol.section_number()) < binary_->sections_.size()) {
-      symbol.section_ = binary_->sections_[symbol.section_number()].get();
+        static_cast<uint32_t>(symbol.section_number()) <= binary_->sections_.size()) {
+      symbol.section_ = binary_->sections_[symbol.section_number()-1].get();
     }
 
     for (uint32_t i = 0; i < raw_symbol.NumberOfAuxSymbols; ++i) {
@@ -602,10 +605,10 @@ ok_error_t Parser::parse_symbols() {
         LIEF_DEBUG("Format 5");
       }
 
-      current_offset += details::STRUCT_SIZES::Symbol16Size;
+      current_offset += details::sizes::SYMBOL_16;
     }
 
-    current_offset += details::STRUCT_SIZES::Symbol16Size;
+    current_offset += details::sizes::SYMBOL_16;
     idx += 1 + raw_symbol.NumberOfAuxSymbols;
     binary_->symbols_.push_back(std::move(symbol));
   }
@@ -614,141 +617,173 @@ ok_error_t Parser::parse_symbols() {
 }
 
 
-//
-// parse Debug
-//
-
 ok_error_t Parser::parse_debug() {
-  LIEF_DEBUG("== Parsing Debug ==");
+  LIEF_DEBUG("Parsing debug directory");
 
-  binary_->has_debug_ = true;
+  DataDirectory* dir = binary_->data_directory(DataDirectory::TYPES::DEBUG_DIR);
+  if (dir == nullptr) {
+    return make_error_code(lief_errors::not_found);
+  }
 
-  uint32_t debug_rva    = binary_->data_directory(DATA_DIRECTORY::DEBUG).RVA();
-  uint32_t debug_offset = binary_->rva_to_offset(debug_rva);
-  uint32_t debug_size   = binary_->data_directory(DATA_DIRECTORY::DEBUG).size();
+  const uint32_t debug_rva = dir->RVA();
+  uint32_t debug_off       = binary_->rva_to_offset(debug_rva);
+  const uint32_t debug_sz  = dir->size();
+  const uint32_t debug_end = debug_off + debug_sz;
 
-  for (size_t i = 0; (i + 1) * sizeof(details::pe_debug) <= debug_size; i++) {
-    auto res_debug_struct = stream_->peek<details::pe_debug>(debug_offset + i * sizeof(details::pe_debug));
-    if (!res_debug_struct) {
+  stream_->setpos(debug_off);
+  while (stream_->pos() < debug_end) {
+    auto res = stream_->read<details::pe_debug>();
+    if (!res) {
       break;
     }
-    binary_->debug_.push_back(*res_debug_struct);
-
-    DEBUG_TYPES type = binary_->debug().back().type();
-
+    const auto type = static_cast<Debug::TYPES>(res->Type);
+    LIEF_DEBUG("Type is: {}", to_string(type));
     switch (type) {
-      case DEBUG_TYPES::IMAGE_DEBUG_TYPE_CODEVIEW:
+      case Debug::TYPES::CODEVIEW:
         {
-          parse_debug_code_view(binary_->debug().back());
+          if (std::unique_ptr<Debug> cv = parse_code_view(*res)) {
+            binary_->debug_.push_back(std::move(cv));
+          } else {
+            LIEF_WARN("Can't parse PE CodeView");
+          }
           break;
         }
 
-      case DEBUG_TYPES::IMAGE_DEBUG_TYPE_POGO:
+      case Debug::TYPES::POGO:
         {
-          parse_debug_pogo(binary_->debug().back());
+          if (std::unique_ptr<Debug> pogo = parse_pogo(*res)) {
+            binary_->debug_.push_back(std::move(pogo));
+          } else {
+            LIEF_WARN("Can't parse PE POGO");
+          }
           break;
         }
 
-      case DEBUG_TYPES::IMAGE_DEBUG_TYPE_REPRO:
+      case Debug::TYPES::REPRO:
         {
-          binary_->is_reproducible_build_ = true;
+          if (std::unique_ptr<Debug> repro = parse_repro(*res)) {
+            binary_->debug_.push_back(std::move(repro));
+          } else {
+            LIEF_WARN("Can't parse PE Repro");
+          }
           break;
         }
 
       default:
-        {}
+        {
+          binary_->debug_.push_back(std::make_unique<Debug>(*res));
+          break;
+        }
     }
   }
   return ok();
 }
 
-ok_error_t Parser::parse_debug_code_view(Debug& debug_info) {
+std::unique_ptr<Debug> Parser::parse_code_view(const details::pe_debug& debug_info) {
   LIEF_DEBUG("Parsing Debug Code View");
+  const uint32_t debug_off = debug_info.PointerToRawData;
 
-  const uint32_t debug_off = debug_info.pointerto_rawdata();
-  auto res_sig = stream_->peek<uint32_t>(debug_off);
+  ScopedStream sscoped(*stream_, debug_off);
+
+  auto res_sig = sscoped->peek<uint32_t>(debug_off);
   if (!res_sig) {
-    return res_sig.error();
+    return nullptr;
   }
 
-  const auto signature = static_cast<CODE_VIEW_SIGNATURES>(*res_sig);
+  const auto signature = static_cast<CodeView::SIGNATURES>(*res_sig);
+  auto default_value = std::make_unique<CodeView>(debug_info, signature);
 
   switch (signature) {
-    case CODE_VIEW_SIGNATURES::CVS_PDB_70:
+    case CodeView::SIGNATURES::PDB_70:
       {
-        const auto pdb_s = stream_->peek<details::pe_pdb_70>(debug_off);
+        const auto pdb_s = sscoped->read<details::pe_pdb_70>();
         if (!pdb_s) {
-          break;
+          return default_value;
         }
 
-        CodeViewPDB::signature_t sig;
-        std::move(std::begin(pdb_s->signature), std::end(pdb_s->signature), std::begin(sig));
-
-        auto res_path = stream_->peek_string_at(debug_off + offsetof(details::pe_pdb_70, filename));
-        if (res_path) {
-          auto codeview = std::make_unique<CodeViewPDB>(CodeViewPDB::from_pdb70(sig, pdb_s->age, *res_path));
-          debug_info.code_view_ = std::move(codeview);
+        auto cv_pdb70 = std::make_unique<CodeViewPDB>(debug_info, *pdb_s);
+        if (auto fname = stream_->read_string()) {
+          cv_pdb70->filename(std::move(*fname));
         }
-        break;
+        return cv_pdb70;
       }
 
     default:
       {
-        LIEF_INFO("Signature {} is not implemented yet!", to_string(signature));
+        LIEF_INFO("CodeView signature '{}' is not implemented yet!",
+                  to_string(signature));
       }
   }
-  return ok();
+  return default_value;
 }
 
-ok_error_t Parser::parse_debug_pogo(Debug& debug_info) {
-  LIEF_DEBUG("== Parsing Debug POGO ==");
+std::unique_ptr<Debug> Parser::parse_pogo(const details::pe_debug& debug_info) {
+  LIEF_DEBUG("Parsing POGO");
+  const uint32_t debug_size = debug_info.SizeOfData;
+  const uint32_t debug_off  = debug_info.PointerToRawData;
+  const uint32_t debug_end  = debug_off + debug_size;
 
-  const uint32_t debug_size = debug_info.sizeof_data();
-  const uint32_t debug_off  = debug_info.pointerto_rawdata();
+  ScopedStream sscoped(*stream_, debug_off);
 
-  auto res_sig = stream_->peek<uint32_t>(debug_off);
+  auto res_sig = sscoped->read<uint32_t>();
   if (!res_sig) {
-    return res_sig.error();
+    return nullptr;
   }
-  const auto signature = static_cast<POGO_SIGNATURES>(*res_sig);
+
+  const auto signature = static_cast<Pogo::SIGNATURES>(*res_sig);
+  auto pogo = std::make_unique<Pogo>(debug_info, signature);
 
   switch (signature) {
-    case POGO_SIGNATURES::POGO_LCTG:
+    case Pogo::SIGNATURES::ZERO: // zero-signature may contain valid entries
+    case Pogo::SIGNATURES::LCTG:
       {
-        auto pogo_object = std::make_unique<Pogo>();
-        pogo_object->signature_ = signature;
-
-        uint32_t offset = sizeof(uint32_t);
-        while (offset + sizeof(details::pe_pogo) < debug_size) {
-          auto res_pogo = stream_->peek<details::pe_pogo>(debug_off + offset);
-          auto res_name = stream_->peek_string_at(debug_off + offset + offsetof(details::pe_pogo, name));
-          if (!res_pogo || !res_name) {
+        while (sscoped->pos() < debug_end) {
+          auto raw = sscoped->read<details::pe_pogo>();
+          if (!raw) {
             break;
           }
 
-          PogoEntry entry;
+          PogoEntry entry{raw->start_rva, raw->size};
+          if (auto name = sscoped->read_string()) {
+            entry.name(std::move(*name));
+          }
 
-          entry.start_rva_ = res_pogo->start_rva;
-          entry.size_      = res_pogo->size;
-          entry.name_      = std::move(*res_name);
-
-          // pogo entries are 4-bytes aligned
-          offset += offsetof(details::pe_pogo, name) + entry.name_.length() + 1;
-          offset += ((4 - offset) % 4);
-
-          pogo_object->entries_.push_back(std::move(entry));
+          pogo->add(std::move(entry));
+          sscoped->align(4);
         }
 
-        debug_info.pogo_ = std::move(pogo_object);
-        break;
+        return pogo;
       }
 
+    case Pogo::SIGNATURES::UNKNOWN:
     default:
       {
         LIEF_INFO("PGO with signature 0x{:x} is not implemented yet!", *res_sig);
       }
   }
-  return ok();
+  return pogo;
+}
+
+std::unique_ptr<Debug> Parser::parse_repro(const details::pe_debug& debug_info) {
+  LIEF_DEBUG("Parsing Debug Repro");
+  const uint32_t debug_off  = debug_info.PointerToRawData;
+
+  /* const uint32_t debug_size = debug_info.SizeOfData; */
+  /* const uint32_t debug_end  = debug_off + debug_size; */
+  ScopedStream sscoped(*stream_, debug_off);
+
+  auto res_size = sscoped->read<uint32_t>();
+  if (!res_size) {
+    return nullptr;
+  }
+  LIEF_DEBUG("Size: 0x{:x}", *res_size);
+
+  std::vector<uint8_t> hash;
+  if (!sscoped->read_data(hash, *res_size)) {
+    LIEF_INFO("Can't read debug reproducible build hash");
+  }
+
+  return std::make_unique<Repro>(debug_info, std::move(hash));
 }
 
 
@@ -785,11 +820,8 @@ inline result<uint32_t> name_table_value(BinaryStream& stream,
 }
 
 
-//
-// Parse Export
-//
 ok_error_t Parser::parse_exports() {
-  LIEF_DEBUG("== Parsing exports ==");
+  LIEF_DEBUG("Parsing exports");
   static constexpr uint32_t NB_ENTRIES_LIMIT   = 0x1000000;
   static constexpr size_t MAX_EXPORT_NAME_SIZE = 4096; // Because of C++ mangling
 
@@ -797,10 +829,14 @@ ok_error_t Parser::parse_exports() {
     uint32_t start;
     uint32_t end;
   };
-  const DataDirectory& export_dir = binary_->data_directory(DATA_DIRECTORY::EXPORT_TABLE);
+  const DataDirectory* export_dir = binary_->data_directory(DataDirectory::TYPES::EXPORT_TABLE);
 
-  uint32_t exports_rva    = export_dir.RVA();
-  uint32_t exports_size   = export_dir.size();
+  if (export_dir == nullptr) {
+    return make_error_code(lief_errors::not_found);
+  }
+
+  uint32_t exports_rva    = export_dir->RVA();
+  uint32_t exports_size   = export_dir->size();
   uint32_t exports_offset = binary_->rva_to_offset(exports_rva);
   range_t range = {exports_rva, exports_rva + exports_size};
 
@@ -813,19 +849,19 @@ ok_error_t Parser::parse_exports() {
     return make_error_code(lief_errors::read_error);
   }
 
-  Export export_object = export_dir_tbl;
+  auto export_object = std::make_unique<Export>(export_dir_tbl);
   uint32_t name_offset = binary_->rva_to_offset(export_dir_tbl.NameRVA);
   if (auto res_name = stream_->peek_string_at(name_offset, Parser::MAX_DLL_NAME_SIZE)) {
     std::string name = *res_name;
-    if (Parser::is_valid_dll_name(name)) {
-      export_object.name_ = std::move(name);
-      LIEF_DEBUG("Export name {}@0x{:x}", export_object.name_, name_offset);
+    if (is_valid_dll_name(name)) {
+      export_object->name_ = std::move(name);
+      LIEF_DEBUG("Export name {}@0x{:x}", export_object->name_, name_offset);
     } else {
-      // Empty export names are not allowed
       if (name.empty()) {
-        return make_error_code(lief_errors::corrupted);
+        LIEF_DEBUG("Export name is empty");
+      } else {
+        LIEF_DEBUG("'{}' is not a valid export name", printable_string(name));
       }
-      LIEF_DEBUG("'{}' is not a valid export name", printable_string(name));
     }
   } else {
     LIEF_INFO("DLL name seems corrupted");
@@ -943,34 +979,54 @@ ok_error_t Parser::parse_exports() {
 
       // Split on '.'
       const size_t dot_pos = fwd_str.find('.');
+      bool is_valid_fwd = false;
       if (dot_pos != std::string::npos) {
         library  = fwd_str.substr(0, dot_pos);
         function = fwd_str.substr(dot_pos + 1);
+        is_valid_fwd = true;
       }
       entry.set_forward_info(std::move(library), std::move(function));
+
+      if (is_valid_fwd) {
+        if (auto name_rva = name_table_value(*stream_, name_table_offset, i)) {
+          uint32_t name_offset = binary_->rva_to_offset(*name_rva);
+          if (auto name = stream_->peek_string_at(name_offset)) {
+            const bool is_valid = !name->empty() &&
+              name->size() <= MAX_EXPORT_NAME_SIZE && is_printable(*name);
+            if (is_valid) {
+              entry.name_ = std::move(*name);
+            }
+          }
+        }
+      }
     }
   }
 
   for (ExportEntry& entry : export_entries) {
     if (corrupted_entries.count(entry.ordinal()) == 0) {
-      export_object.entries_.push_back(std::move(entry));
+      export_object->entries_.push_back(std::move(entry));
     }
   }
 
   binary_->export_ = std::move(export_object);
-  binary_->has_exports_ = true;
   return ok();
 }
 
 ok_error_t Parser::parse_signature() {
-  LIEF_DEBUG("== Parsing signature ==");
+  LIEF_DEBUG("Parsing signature");
   static constexpr size_t SIZEOF_HEADER = 8;
 
   /*** /!\ In this data directory, RVA is used as an **OFFSET** /!\ ****/
   /*********************************************************************/
-  const uint32_t signature_offset  = binary_->data_directory(DATA_DIRECTORY::CERTIFICATE_TABLE).RVA();
-  const uint32_t signature_size    = binary_->data_directory(DATA_DIRECTORY::CERTIFICATE_TABLE).size();
+  const DataDirectory* cert_dir = binary_->data_directory(DataDirectory::TYPES::CERTIFICATE_TABLE);
+  if (cert_dir == nullptr) {
+    return make_error_code(lief_errors::not_found);
+  }
+
+  const uint32_t signature_offset  = cert_dir->RVA();
+  const uint32_t signature_size    = cert_dir->size();
   const uint64_t end_p = signature_offset + signature_size;
+
   LIEF_DEBUG("Signature Offset: 0x{:04x}", signature_offset);
   LIEF_DEBUG("Signature Size:   0x{:04x}", signature_size);
 
@@ -985,7 +1041,7 @@ ok_error_t Parser::parse_signature() {
     if (auto res = stream_->read<uint32_t>()) {
       length = *res;
     } else {
-      return res.error();
+      return make_error_code(res.error());
     }
 
     if (length <= SIZEOF_HEADER) {
@@ -1030,7 +1086,7 @@ ok_error_t Parser::parse_signature() {
 
 
 ok_error_t Parser::parse_overlay() {
-  LIEF_DEBUG("== Parsing Overlay ==");
+  LIEF_DEBUG("Parsing Overlay");
   const uint64_t last_section_offset = std::accumulate(
       std::begin(binary_->sections_), std::end(binary_->sections_), uint64_t{ 0u },
       [] (uint64_t offset, const std::unique_ptr<Section>& section) {
@@ -1050,82 +1106,34 @@ ok_error_t Parser::parse_overlay() {
   return ok();
 }
 
-
-result<uint32_t> Parser::checksum() {
-  /*
-   * (re)compute the checksum specified in OptionalHeader::CheckSum
-   */
-  ScopedStream chk_stream(*stream_, 0);
-  const uint32_t padding = stream_->size() % sizeof(uint16_t);
-
-  LIEF_DEBUG("padding: {}", padding);
-
-  uint32_t partial_sum = 0;
-  const uint64_t file_length = stream_->size();
-  uint64_t nb_chunk = (file_length + 1) >> 1; // Number of uint16_t chunks
-
-  while (*stream_) {
-    uint16_t chunk = 0;
-    if (auto res = stream_->read<uint16_t>()) {
-      chunk = *res;
-    } else {
-      break;
-    }
-    --nb_chunk;
-    partial_sum += chunk;
-    partial_sum = (partial_sum >> 16) + (partial_sum & 0xffff);
-  }
-
-  if (nb_chunk > 0) {
-    if (auto res = stream_->read<uint8_t>()) {
-      partial_sum += *res;
-      partial_sum = (partial_sum >> 16) + (partial_sum & 0xffff);
-    }
-  }
-
-  auto partial_sum_res = static_cast<uint16_t>(((partial_sum >> 16) + partial_sum) & 0xffff);
-  uint32_t binary_checksum = binary_->optional_header().checksum();
-  uint32_t adjust_sum_lsb = binary_checksum & 0xFFFF;
-  uint32_t adjust_sum_msb = binary_checksum >> 16;
-
-  partial_sum_res -= (partial_sum_res < adjust_sum_lsb);
-  partial_sum_res -= adjust_sum_lsb;
-
-  partial_sum_res -= (partial_sum_res < adjust_sum_msb);
-  partial_sum_res -= adjust_sum_msb;
-
-  return static_cast<uint32_t>(partial_sum_res) + file_length;
-}
-
-//
-// Return the Binary constructed
-//
-std::unique_ptr<Binary> Parser::parse(const std::string& filename) {
+std::unique_ptr<Binary> Parser::parse(const std::string& filename,
+                                      const ParserConfig& conf) {
   if (!is_pe(filename)) {
     return nullptr;
   }
   Parser parser{filename};
-  parser.init(filename);
+  parser.init(conf);
   return std::move(parser.binary_);
 }
 
-
-std::unique_ptr<Binary> Parser::parse(std::vector<uint8_t> data, const std::string& name) {
+std::unique_ptr<Binary> Parser::parse(std::vector<uint8_t> data,
+                                      const ParserConfig& conf) {
   if (!is_pe(data)) {
     return nullptr;
   }
   Parser parser{std::move(data)};
-  parser.init(name);
+  parser.init(conf);
   return std::move(parser.binary_);
 }
 
-std::unique_ptr<Binary> Parser::parse(std::unique_ptr<BinaryStream> stream, const std::string& name) {
+std::unique_ptr<Binary> Parser::parse(std::unique_ptr<BinaryStream> stream,
+                                      const ParserConfig& conf) {
   if (!is_pe(*stream)) {
     return nullptr;
   }
 
   Parser parser{std::move(stream)};
-  parser.init(name);
+  parser.init(conf);
   return std::move(parser.binary_);
 }
 
@@ -1146,7 +1154,7 @@ bool Parser::is_valid_import_name(const std::string& name) {
 
 
 bool Parser::is_valid_dll_name(const std::string& name) {
-  //! @brief Minimum size for a DLL's name
+  /// @brief Minimum size for a DLL's name
   static constexpr unsigned MIN_DLL_NAME_SIZE = 4;
 
   if (name.size() < MIN_DLL_NAME_SIZE || name.size() > Parser::MAX_DLL_NAME_SIZE) {
