@@ -1,5 +1,5 @@
-/* Copyright 2021 - 2023 R. Thomas
- * Copyright 2021 - 2023 Quarkslab
+/* Copyright 2021 - 2024 R. Thomas
+ * Copyright 2021 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,22 +19,6 @@
 namespace LIEF {
 namespace PE {
 
-MsSpcStatementType::MsSpcStatementType() :
-  Attribute(SIG_ATTRIBUTE_TYPES::MS_SPC_STATEMENT_TYPE)
-{}
-
-MsSpcStatementType::MsSpcStatementType(const MsSpcStatementType&) = default;
-MsSpcStatementType& MsSpcStatementType::operator=(const MsSpcStatementType&) = default;
-
-std::unique_ptr<Attribute> MsSpcStatementType::clone() const {
-  return std::unique_ptr<Attribute>(new MsSpcStatementType{*this});
-}
-
-MsSpcStatementType::MsSpcStatementType(oid_t oid) :
-  Attribute(SIG_ATTRIBUTE_TYPES::MS_SPC_STATEMENT_TYPE),
-  oid_{std::move(oid)}
-{}
-
 void MsSpcStatementType::accept(Visitor& visitor) const {
   visitor.visit(*this);
 }
@@ -42,9 +26,5 @@ void MsSpcStatementType::accept(Visitor& visitor) const {
 std::string MsSpcStatementType::print() const {
   return oid() + " (" + oid_to_string(oid()) + ")";
 }
-
-
-MsSpcStatementType::~MsSpcStatementType() = default;
-
 }
 }
