@@ -30,11 +30,19 @@ def test_add_ordinal(tmp_path, test_exe):
     builder = lief.PE.Builder(binary)
     builder.build_imports(True).patch_imports(True)
     builder.build()
-    builder.write(str(new_path))
+    builder.write(new_path.as_posix())
 
-    new_binary = lief.parse(str(new_path))
+    new_binary = lief.parse(new_path.as_posix())
     assert new_binary.has_import(lib_name)
     new_lib = binary.get_import(lib_name)
     first_ord = next(iter([e for e in new_lib.entries if e.is_ordinal]))
     assert first_ord is not None
     assert first_ord.ordinal == ordinal_val
+
+
+def test_resolve_ordinal():
+    pe = lief.PE.parse(get_sample("PE/PE64_x86-64_binary_mfc-application.exe"))
+    imp = pe.get_import("OLEAUT32.dll")
+    new_imp: lief.PE.Import = lief.PE.resolve_ordinals(imp)
+    assert new_imp.entries[0].name == "SysAllocString"
+    assert new_imp.entries[1].name == "VariantClear"
