@@ -1,12 +1,17 @@
 import lief
+import hashlib
+import pytest
+from pathlib import Path
 from utils import get_sample, is_64bits_platform
 
-winhello64 = lief.parse(get_sample('PE/PE64_x86-64_binary_winhello64-mingw.exe'))
-atapi      = lief.parse(get_sample('PE/PE64_x86-64_atapi.sys'))
+winhello64 = lief.PE.parse(get_sample('PE/PE64_x86-64_binary_winhello64-mingw.exe'))
+atapi      = lief.PE.parse(get_sample('PE/PE64_x86-64_atapi.sys'))
 
 def test_dos_header():
     dos_header: lief.PE.DosHeader = atapi.dos_header
 
+    assert dos_header.copy() == dos_header
+
     assert dos_header.addressof_new_exeheader == 0xd8
     assert dos_header.addressof_relocation_table == 0x40
     assert dos_header.checksum == 0x0
@@ -23,26 +28,39 @@ def test_dos_header():
     assert dos_header.oem_id == 0x0
     assert dos_header.oem_info == 0x0
     assert dos_header.overlay_number == 0x0
-    assert dos_header.used_bytes_in_the_last_page == 0x90
+    assert dos_header.used_bytes_in_last_page == 0x90
+
+    assert hashlib.sha256(atapi.dos_stub).hexdigest() == "2e6296653faf1fd51d875fab7e08c38f06f9a7eccb718c569dee5e3041075a6a"
+    print(dos_header)
 
 def test_header():
-    header: lief.PE.Header = atapi.header
+    header = atapi.header
 
+    assert header.copy() == header
     assert header.numberof_sections == 0x6
     assert header.numberof_symbols == 0x0
     assert header.pointerto_symbol_table == 0x0
     assert header.signature == [80, 69, 0, 0]
     assert header.sizeof_optional_header == 0xf0
     assert header.time_date_stamps == 0x4a5bc113
-    assert header.machine == lief.PE.MACHINE_TYPES.AMD64
-    assert header.characteristics_list == {lief.PE.HEADER_CHARACTERISTICS.LARGE_ADDRESS_AWARE, lief.PE.HEADER_CHARACTERISTICS.EXECUTABLE_IMAGE}
+    assert header.machine == lief.PE.Header.MACHINE_TYPES.AMD64
+    assert header.characteristics_list == [
+            lief.PE.Header.CHARACTERISTICS.EXECUTABLE_IMAGE,
+            lief.PE.Header.CHARACTERISTICS.LARGE_ADDRESS_AWARE,
+            ]
+    print(header)
+    assert header.copy() == header
 
 
 def test_optional_header():
-    header: lief.PE.OptionalHeader = atapi.optional_header
+    header = atapi.optional_header
+    print(header)
+
+    assert header.copy() == header
 
     assert header.addressof_entrypoint == 0x7064
     assert header.baseof_code == 0x1000
+    assert header.baseof_data == 0
     assert header.checksum == 0x65bb
     assert header.dll_characteristics == 0x0
     assert header.file_alignment == 0x200
@@ -68,9 +86,9 @@ def test_optional_header():
     assert header.sizeof_stack_commit == 0x1000
     assert header.sizeof_stack_reserve == 0x40000
     assert header.sizeof_uninitialized_data == 0x0
-    assert header.subsystem == lief.PE.SUBSYSTEM.NATIVE
+    assert header.subsystem == lief.PE.OptionalHeader.SUBSYSTEM.NATIVE
     assert header.win32_version_value == 0x0
-
+    assert header.dll_characteristics_lists == []
 
 def test_data_directories():
     dirs = atapi.data_directories
@@ -145,6 +163,9 @@ def test_data_directories():
     assert dirs[15].size == 0x0
     assert not dirs[15].has_section
 
+    assert dirs[1].copy() == dirs[1]
+    print(dirs[1])
+
 def test_sections():
     sections = winhello64.sections
 
@@ -234,6 +255,7 @@ def test_sections():
         assert lief.hash(list(sections[4].padding)) == 6267801405663681729
         assert lief.hash(list(sections[4].content)) == 9792162199629343627
 
+    assert sections[5].copy() == sections[5]
     assert sections[5].name == ".rsrc"
     assert sections[5].virtual_size == 0x3f0
     assert sections[5].virtual_address == 0x8000
@@ -245,6 +267,14 @@ def test_sections():
     assert sections[5].numberof_line_numbers == 0x0
     assert int(sections[5].characteristics) == 0x42000040
 
+    # make sure we can't write more than the size of a name
+    custom_section = lief.PE.Section(".test")
+    custom_section.name = 'a' * 30
+    custom_section.content = [1] * 10
+    assert custom_section.content[0] == 1
+    assert custom_section.name == ".test"
+
+
     if is_64bits_platform():
         assert lief.hash(list(sections[5].padding)) == 2694043916712032187
         assert lief.hash(list(sections[5].content)) == 17560174430803761296
@@ -255,12 +285,18 @@ def test_tls():
     tls = winhello64.tls
 
     assert tls.addressof_callbacks == 0x409040
-    assert tls.callbacks, [0x4019c0 == 0x401990]
+    assert tls.callbacks == [0x4019c0, 0x401990]
     assert tls.addressof_index == 0x4075fc
     assert tls.sizeof_zero_fill == 0
     assert tls.characteristics == 0
-    assert tls.addressof_raw_data, (0x40a000 == 0x40a060)
+    assert tls.addressof_raw_data == (0x40a000, 0x40a060)
     assert tls.section.name == ".tls"
+    assert tls.copy() == tls
+    print(tls)
+
+    assert tls.directory is not None
+    assert tls.directory.type == lief.PE.DataDirectory.TYPES.TLS_TABLE
+    assert hashlib.sha256(tls.data_template).hexdigest() == "ffb6b993bf4ae7ec095d4aeba45ac7e9973e16c17077058260f3f4eb0487d07e"
 
 def test_imports():
     imports  = winhello64.imports
@@ -279,6 +315,10 @@ def test_imports():
     assert entry_12.hint == 0x34b
     assert entry_12.iat_value == 0x84ba
     assert entry_12.iat_address == 0x825c
+    assert kernel32.get_entry("DoesNotExist") is None
+    assert kernel32.get_entry("LeaveCriticalSection") == entry_12
+    assert kernel32.get_entry("LeaveCriticalSection") != kernel32.entries[11].copy()
+    assert kernel32.get_function_rva_from_iat("LeaveCriticalSection") == 96
 
     msvcrt = imports[1]
     assert msvcrt.name == "msvcrt.dll"
@@ -293,6 +333,11 @@ def test_imports():
     assert entry_0.iat_value == 0x85ca
     assert entry_0.iat_address == 0x82cc
 
+    assert msvcrt.directory == winhello64.data_directory(lief.PE.DataDirectory.TYPES.IMPORT_TABLE)
+    assert msvcrt.iat_directory == winhello64.data_directory(lief.PE.DataDirectory.TYPES.IAT)
+    assert msvcrt.get_function_rva_from_iat("") == lief.lief_errors.not_found
+    assert msvcrt.get_function_rva_from_iat("__C_specific_handler") == 0
+
 def test_issue_imports():
     pe: lief.PE.Binary = lief.parse(get_sample("PE/abdce8577b46e4e23346f06ba8b9ab05cf47e92aec7e615c04436301355cd86d.pe"))
     imports = pe.imports
@@ -327,18 +372,30 @@ def test_issue_exports():
     assert not exports.entries[6].is_extern
     assert exports.entries[6].function_rva == 0x203a0
 
+    assert exports.entries[6].value == 0x203a0
+
+    entry = exports.entries[6]
+    assert not entry.is_forwarded
+    assert entry.forward_information.function == ""
+    assert entry.forward_information.library == ""
+
+    assert exports.copy() == exports
+
 def test_issue_685():
     """
     https://github.com/lief-project/LIEF/issues/685
     """
 
-    pe: lief.PE.Binary = lief.parse(get_sample("PE/2420feb9d03efc1aa07b4117390c29cd8fee826ea1b48fee89660d65a3a8ba2b.neut"))
+    pe = lief.PE.parse(get_sample("PE/2420feb9d03efc1aa07b4117390c29cd8fee826ea1b48fee89660d65a3a8ba2b.neut"))
 
     exports = pe.get_export()
 
     assert exports.name == "nmv.ocx"
     assert exports.entries[0].name == "oplk"
 
+    entry = exports.entries[0]
+    assert entry is not None
+
 def test_rich_header():
     rheader = atapi.rich_header
     assert rheader.key == 0xa476a6e3
@@ -352,16 +409,23 @@ def test_rich_header():
     assert entry_4.build_id == 0x7809
     assert entry_4.count == 1
     hex_val = bytes(rheader.raw(rheader.key)).hex()
-    assert hex_val == "a7c718f7e3a676a4e3a676a4e3a676a4" \
-                      "eadee5a4e6a676a4e3a677a4fba676a4" \
-                      "eadee3a4e2a676a4eadef5a4e0a676a4" \
-                      "eadeffa4e1a676a4eadee2a4e2a676a4" \
-                      "eadee7a4e2a676a452696368e3a676a4"
+    assert hex_val == ("a7c718f7e3a676a4e3a676a4e3a676a4"
+                       "eadee5a4e6a676a4e3a677a4fba676a4"
+                       "eadee3a4e2a676a4eadef5a4e0a676a4"
+                       "eadeffa4e1a676a4eadee2a4e2a676a4"
+                       "eadee7a4e2a676a452696368e3a676a4")
 
     sha256 = bytes(rheader.hash(lief.PE.ALGORITHMS.SHA_256, rheader.key)).hex()
     assert sha256 == "1bda7d55023ff27b0ea1c9f56d53ca77ca4264ac58fdee8daac58cdc060bf2da"
 
-
+    assert rheader.copy() == rheader
+    print(rheader)
+    new_entry = lief.PE.RichEntry(1, 2, 3)
+    print(new_entry)
+    rheader.add_entry(new_entry)
+    assert rheader.entries[-1] == new_entry
+    assert rheader.entries[-1].copy() == new_entry
+    assert lief.PE.RichEntry() == lief.PE.RichEntry(0, 0, 0)
 
 def test_relocations():
     pe: lief.PE.Binary = lief.parse(get_sample("PE/PE64_x86-64_binary_mfc-application.exe"))
@@ -371,18 +435,20 @@ def test_relocations():
     assert len(relocations[0].entries) == 88
     relocation = relocations[0]
 
+    assert relocation.entries[46].size == 64
+    assert relocation.entries[46].address == 0xdeb8
     assert relocation.entries[46].data == 0xaeb8
     assert relocation.entries[46].position == 0xeb8
-    assert relocation.entries[46].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[46].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[25].data == 0xae10
     assert relocation.entries[25].position == 0xe10
-    assert relocation.entries[25].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[25].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[56].data == 0xaf08
     assert relocation.entries[56].position == 0xf08
-    assert relocation.entries[56].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[56].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[75].data == 0xafa0
     assert relocation.entries[75].position == 0xfa0
-    assert relocation.entries[75].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[75].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
 
     assert relocations[8].virtual_address == 0x15000
     assert relocations[8].block_size == 0xc0
@@ -390,16 +456,16 @@ def test_relocations():
     relocation = relocations[8]
     assert relocation.entries[87].data == 0xa9f8
     assert relocation.entries[87].position == 0x9f8
-    assert relocation.entries[87].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[87].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[24].data == 0xa0c0
     assert relocation.entries[24].position == 0xc0
-    assert relocation.entries[24].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[24].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[67].data == 0xa218
     assert relocation.entries[67].position == 0x218
-    assert relocation.entries[67].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[67].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[54].data == 0xa1b0
     assert relocation.entries[54].position == 0x1b0
-    assert relocation.entries[54].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[54].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
 
     assert relocations[9].virtual_address == 0x1c000
     assert relocations[9].block_size == 0x80
@@ -407,20 +473,40 @@ def test_relocations():
     relocation = relocations[9]
     assert relocation.entries[40].data == 0xa628
     assert relocation.entries[40].position == 0x628
-    assert relocation.entries[40].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[40].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[17].data == 0xa2d8
     assert relocation.entries[17].position == 0x2d8
-    assert relocation.entries[17].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[17].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[36].data == 0xa5a0
     assert relocation.entries[36].position == 0x5a0
-    assert relocation.entries[36].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[36].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
     assert relocation.entries[52].data == 0xa7f8
     assert relocation.entries[52].position == 0x7f8
-    assert relocation.entries[52].type == lief.PE.RELOCATIONS_BASE_TYPES.DIR64
+    assert relocation.entries[52].type == lief.PE.RelocationEntry.BASE_TYPES.DIR64
+
+    r1 = lief.PE.RelocationEntry()
+    r1.position = 0x123
+    r1.type = lief.PE.RelocationEntry.BASE_TYPES.LOW
+
+    assert r1.address == r1.position
+    assert r1.size == 16
+
+    r2 = lief.PE.RelocationEntry()
+    r2.type = lief.PE.RelocationEntry.BASE_TYPES.HIGHLOW
+    assert r2.size == 32
+
+    r3 = lief.PE.RelocationEntry()
+    r3.type = lief.PE.RelocationEntry.BASE_TYPES.ABS
+    assert r3.size == 0
+    r3.data = 0xBAAA
+    assert r3.position == 0xAAA
+    assert r3.type == 0xB
+    assert r2 != r3
+
+    assert relocation.copy() == relocation
 
 
 def test_symbols():
-
     symbols = winhello64.symbols
     assert len(symbols) == 1097
 
@@ -430,7 +516,60 @@ def test_symbols():
     assert symbol.section_number == 1
     assert symbol.type == 32
 
+@pytest.mark.parametrize("input_file", [
+    "PE/PE64_x86-64_binary_winhello64-mingw.exe",
+    "PE/PE32_x86_binary_KMSpico_setup_MALWARE.exe",
+    "PE/PE64_x86-64_atapi.sys",
+    "PE/PE32_x86-64_binary_avast-free-antivirus-setup-online.exe"
+])
+def test_checksum(input_file: str):
+    pe = lief.PE.parse(get_sample(input_file))
+    assert pe.compute_checksum() == pe.optional_header.checksum
 
-def test_checksum():
-    assert atapi.optional_header.computed_checksum      == atapi.optional_header.checksum
-    assert winhello64.optional_header.computed_checksum == winhello64.optional_header.checksum
+def test_config():
+    config = lief.PE.ParserConfig()
+    config.parse_signature = False
+    config.parse_imports = False
+    config.parse_rsrc = False
+    config.parse_reloc = False
+
+    fpath = get_sample("PE/PE32_x86-64_binary_avast-free-antivirus-setup-online.exe")
+    avast = lief.PE.parse(fpath, config)
+
+    assert len(avast.imports) == 0
+    assert len(avast.relocations) == 0
+    assert len(avast.signatures) == 0
+    assert avast.resources is None
+
+def test_overlay():
+    pe = lief.PE.parse(get_sample("PE/PE32_x86_binary_KMSpico_setup_MALWARE.exe"))
+    assert len(pe.overlay) == 3073728
+    assert hashlib.sha256(pe.overlay).hexdigest() == "01c0472ead112b44dca6996c9fae47e0d6870e61792ef606ea47067932115d01"
+
+def test_path_like():
+    assert lief.PE.parse(Path(get_sample('PE/PE32_x86-64_binary_avast-free-antivirus-setup-online.exe'))) is not None
+
+def test_issue_1049():
+    pe = lief.PE.parse(get_sample("PE/issue_1049.exe"))
+    assert pe is not None
+
+def test_xbox_file():
+    pe = lief.PE.parse(get_sample("PE/backcompat.exe"))
+    assert pe.header.machine == lief.PE.Header.MACHINE_TYPES.POWERPCBE
+
+def test_issue_1115():
+    """
+    Infinite loop in PE resource tree
+    """
+    pe = lief.PE.parse(get_sample("PE/issue_1115.pe"))
+    assert pe is not None
+
+def test_large_ordinal():
+    """
+    Issue coming from Goblin project: goblin/issues/428
+    """
+    pe = lief.PE.parse(get_sample("PE/special_import_forwarder_tls.exe.bin"))
+    imp = pe.imports[0]
+    assert imp.name == "abcd.dll"
+
+    assert imp.entries[0].ordinal == 0xc8c6
